#!/usr/bin/env python3
import os
import sys
import time
from pathlib import Path
from subprocess import run, PIPE, call, check_call, CalledProcessError
from typing import Optional

os.environ["FWUPD_MSS"] = "255"
# os.environ["DRIVER_NAME"] = "ESTKme-RED"

BASE_PATH = Path(__file__).parent
SHA1SUMS_PATH = BASE_PATH / "SHA1SUMS"
FILES_PATH = BASE_PATH / "files"
VERSION_PATH = BASE_PATH / "VERSION"


class FirmwareUpgrader:
    def __init__(self):
        uname = os.uname()
        if uname.sysname == "Darwin":
            filename = "fwupd.darwin"
        elif uname.sysname == "Linux":
            filename = f"fwupd.linux-{uname.machine}"
        else:
            raise Exception(f"Unsupported system: {uname.sysname}")
        self.program = BASE_PATH / "bin" / filename
        self.program.chmod(0o755)

    def get_bootloader_version(self) -> str:
        print("$", self.program.stem, "ver", file=sys.stderr)
        process = run([str(self.program), "ver"], stdout=PIPE, universal_newlines=True)
        return process.stdout.strip()

    def upgrade_bootloader(self, source: str, target: str, filename: Optional[str] = None):
        filename = filename or f"{source}-{target}"
        print(f"Upgrade bootloader from {source} to {target}")
        if source.startswith("T001") and int(source[5:7]) < 5:
            self.upgrade_v1(filename)
        else:
            self.upgrade_v2(filename)

    def upgrade_v1(self, firmware: str):
        call([str(self.program), "v1-dfu", str(FILES_PATH / firmware)])

    def upgrade_v2(self, firmware: str):
        call([str(self.program), "v2-dfu", str(FILES_PATH / firmware)])


def integrity_check():
    if not SHA1SUMS_PATH.exists():
        print("Now the integrity verification been BROKEN, upgrading is RISKY.")
        confirm_text = input("Continue upgrading? (Y/N) ").upper()
        if confirm_text not in {"Y", "YES"}:
            raise Exception("\033[0;31mUpgrade Cancelled\033[0m")
        return
    try:
        args = ["shasum", "--algorithm", "1", "--quiet", "--check", str(SHA1SUMS_PATH)]
        check_call(args, cwd=str(BASE_PATH))
    except CalledProcessError:
        raise Exception("\033[0;31mIntegrity Check: FAILED\033[0m")


def follow_latest():
    upgrader = FirmwareUpgrader()
    while True:
        version = upgrader.get_bootloader_version()
        print("Current bootloader version:", version)
        if version == "T001V02":
            upgrader.upgrade_bootloader(version, "T001V05")
        elif version in ["T001V03", "T001V04"]:
            upgrader.upgrade_bootloader(version, "T001V05", "T001V03_V04-T001V05")
        elif version == "T001V05":
            upgrader.upgrade_bootloader(version, "T001V06")
        elif version == "T002V01":
            upgrader.upgrade_bootloader(version, "T002V06")
        elif version in ["T000V06", "T001V06", "T002V06"]:
            firmware_version = VERSION_PATH.read_text().strip()
            print(f"Upgrade {version} ({firmware_version})")
            upgrader.upgrade_v2(f"{version}-{firmware_version}")
            break
        else:
            raise Exception(f"Unknown bootloader version: {version}")
        time.sleep(3)


def main():
    follow_latest()


if __name__ == "__main__":
    try:
        integrity_check()
        main()
    except KeyboardInterrupt:
        pass
    except Exception as e:
        print(e)
